const fs = require('fs');
const printf = require('printf');
const Parser = require('junit2json').parse;
const basename = require('basename');

const Report = require('../lib/report.js')

class JUnit {
  constructor(options) {
    this.options = options;
    this.file = options.file;
    this.data = options.data;
    this.exclude = options.exclude;
    if (!this.exclude)
      this.exclude = [];
  }

  parseFrames(input) {
    /* Look at every line and guess what is a line number. */
    let lines = input.split("\n");
    let frames = [];

    for (let i = 0; i < lines.length; i++) {
      let m = lines[i].match(/at ([A-za-z]+.*) \((.*\:[0-9]+.*)\)/);

      if (!m || !m[1] || !m[2])
        continue;

      let function_name = m[1];
      let source = m[2];
      let source_file = m[2].split(":");

      let frame = {};

      frame['function'] = function_name;

      if (source_file) {
        frame.file = source_file[0];
        frame.path = source_file[0];
      }

      if (source_file) {
        if (source_file.length >= 2)
          frame.line = parseInt(source_file[1]);
        if (source_file.length >= 3)
          frame.column = parseInt(source_file[2]);
      }

      frames.push(frame);
    }

    if (frames.length === 0) {
      frames.push({
        "function" : "Test case"
      });
    }

    return frames;
  }

  createReport(r, tc, ts) {
    let self = this;

    if (tc.failure && tc.failure.length > 0) {
      for (let i = 0; i < tc.failure.length; i++) {
        let report = new Report(r);
        let f = tc.failure[i];

        if (tc.classname)
          report.set('attribute', 'unit.message', tc.classname);

        if (ts.name) {
          report.set('attribute', 'unit', ts.name);
          if (ts.file) {
            if (ts.file.endsWith(".js"))
              report.set('attribute', 'unit', ts.name + ".js");

            let category_token = ts.name.split("/");
            if (category_token.length > 1)
              report.set('attribute', 'category', category_token[0]);
          }
        }

        if (f.message)
          report.set('attribute', 'error.message', f.message);

        if (f.type)
          report.add('classifiers', f.type);

        if (tc.time)
          report.set('attribute', 'duration', parseInt(tc.time * 1000));

        /* Synthesize a thread and a raw log buffer. */
        if (f.inner) {
          let thread = report.thread("junit");

          /*
           * parseFrames is guaranteed to return at least 1 frame, 
           * even if it's completely synthetic. We use this to attach
           * additional context. Finally, mela is responsible for submitting
           * a fingerprint value.
           */
          let frames = self.parseFrames(f.inner);
          for (let k = 0; k < frames.length; k++)
            thread.frame(frames[k]);

          let faulting_frame = frames[0];
          faulting_frame.text = f.inner;
        }
      }
    }
  }

  parseData(options, cb) {
    var self = this;
    var report = new Report();

    report.checksum(self.data);

    report.add('classifiers', 'junit');

    Parser(self.data).then(function(r) {
      /* Extract failures. */
      if (!r.failures || !r.testsuite || r.testsuite.length === 0)
        return cb(new Error("no testsuite results found"), null);

      for (let i = 0; i < r.testsuite.length; i++) {
        let ts_r = r.testsuite[i];

        if (ts_r.failures === 0)
          continue;

        for (let j = 0; j < ts_r.testcase.length; j++) {
          if (ts_r.testcase[j].failures === 0 || !ts_r.testcase[j].failure)
            continue;

          self.createReport(report, ts_r.testcase[j], ts_r);
        }
      }

      return cb(null, report);
    }).catch(function(e) {
      return cb(e, report);
    });
  }

  parse(options, cb) {
    var self = this;

    if (self.data) {
      return self.parseData(options, cb);
    } else if (self.file) {
      fs.readFile(this.file, "utf8", function(e, d) {
        if (e) {
          return cb(e, null);
        }

        self.data = d;
        return self.parseData(options, cb);
      });
    } else {
      return cb(new Error("no data was supplied"));
    }
  }
}

module.exports = JUnit;

//-- vim:ts=2:et:sw=2:st=2
